<?php
// $Id: pninit.php,v 1.3 2002/03/09 08:24:24 jgm Exp $
// ----------------------------------------------------------------------
// POST-NUKE Content Management System
// Copyright (C) 2002 by the PostNuke Development Team.
// http://www.postnuke.com/
// ----------------------------------------------------------------------
// Based on:
// PHP-NUKE Web Portal System - http://phpnuke.org/
// Thatware - http://thatware.org/
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WIthOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// ----------------------------------------------------------------------
// Author: Mike Welch
// Based on Template Modlue by Jim McDonald
// Purpose of file:  Reco initialisation modules
// ----------------------------------------------------------------------

/**
 * initialise the Reco module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 */
function reco_init()
{
    // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
    // return arrays but we handle them differently.  For pnDBGetConn()
    // we currently just want the first item, which is the official
    // database handle.  For pnDBGetTables() we want to keep the entire
    // tables array together for easy reference later on
    list($dbconn) = pnDBGetConn();
    $pntable = pnDBGetTables();

    // It's good practice to name the table and column definitions you
    // are getting - $table and $column don't cut it in more complex
    // modules
    $recoitemtable = $pntable['recoitem'];
    $recoitemcolumn = &$pntable['recoitem_column'];
    
	$recocategorytable = $pntable['recocategory'];
    $recocategorycolumn = &$pntable['recocategory_column'];

    // Create the table - the formatting here is not mandatory, but it does
    // make the SQL statement relatively easy to read.  Also, separating out
    // the SQL statement from the Execute() command allows for simpler
    // debug operation if it is ever needed
    $sql = "CREATE TABLE $recoitemtable (
            $recoitemcolumn[id] int(10) NOT NULL auto_increment,
            $recoitemcolumn[desc1] varchar(32) NOT NULL default '',
            $recoitemcolumn[desc2] varchar(32) NOT NULL default '',
            $recoitemcolumn[imgsrc] varchar(100) NOT NULL default '',
            $recoitemcolumn[buyurl] varchar(100) NOT NULL default '',
            $recoitemcolumn[reviewtxt] text ,
            $recoitemcolumn[category] int(5) NOT NULL default 0 ,
            PRIMARY KEY(pn_id))";
    $dbconn->Execute($sql);


    // Check for an error with the database code, and if so set an
    // appropriate error message and return
    if ($dbconn->ErrorNo() != 0) {
        pnSessionSetVar('errormsg', _CREATETABLEFAILED);
        return false;
    }

    $sql = "CREATE TABLE $recocategorytable (
            $recocategorycolumn[id] int(10) NOT NULL auto_increment,
            $recocategorycolumn[category] varchar(32) NOT NULL default '',
            PRIMARY KEY(pn_id))";
    $dbconn->Execute($sql);

    if ($dbconn->ErrorNo() != 0) {
        pnSessionSetVar('errormsg', _CREATETABLEFAILED);
        return false;
    }

    // Set up an initial value for a module variable.  Note that all module
    // variables should be initialised with some value in this way rather
    // than just left blank, this helps the user-side code and means that
    // there doesn't need to be a check to see if the variable is set in
    // the rest of the code as it always will be
    pnModSetVar('Reco', 'blocktrailer', '');
    pnModSetVar('Reco', 'pagetrailer', '');
    pnModSetVar('Reco', 'columnsonpage', 3);
    pnModSetVar('Reco', 'clickheretxt', 'Click here to buy from Amazon');


    // Initialisation successful
    return true;
}

/**
 * upgrade the Reco module from an old version
 * This function can be called multiple times
 */
function reco_upgrade($oldversion)
{
    return true;
}

/**
 * delete the Reco module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 */
function reco_delete()
{
    // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
    // return arrays but we handle them differently.  For pnDBGetConn()
    // we currently just want the first item, which is the official
    // database handle.  For pnDBGetTables() we want to keep the entire
    // tables array together for easy reference later on
    list($dbconn) = pnDBGetConn();
    $pntable = pnDBGetTables();

    // Drop the table - for such a simple command the advantages of separating
    // out the SQL statement from the Execute() command are minimal, but as
    // this has been done elsewhere it makes sense to stick to a single method
    $sql = "DROP TABLE $pntable[recoitem]";
    $dbconn->Execute($sql);

    // Check for an error with the database code, and if so set an
    // appropriate error message and return
    if ($dbconn->ErrorNo() != 0) {
        // Report failed deletion attempt
        return false;
    }

    $sql = "DROP TABLE $pntable[recocategory]";
    $dbconn->Execute($sql);
    if ($dbconn->ErrorNo() != 0) {
        // Report failed deletion attempt
        return false;
    }

    // Delete any module variables
    pnModDelVar('Reco', 'blocktrailer', '');
    pnModDelVar('Reco', 'pagetrailer', '');
    pnModDelVar('Reco', 'columnsonpage', 3);
    pnModDelVar('Reco', 'clickheretxt', 'Click here to buy from Amazon');

    // Deletion successful
    return true;
}

?>
