<?php
// $Id: pnuserapi.php,v 1.4 2002/03/09 08:24:24 jgm Exp $
// ----------------------------------------------------------------------
// POST-NUKE Content Management System
// Copyright (C) 2002 by the PostNuke Development Team.
// http://www.postnuke.com/
// ----------------------------------------------------------------------
// Based on:
// PHP-NUKE Web Portal System - http://phpnuke.org/
// Thatware - http://thatware.org/
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WIthOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// ----------------------------------------------------------------------
// Author: Mike Welch
// Based on Template Modlue by Jim McDonald
// Purpose of file:  Reco User API
// ----------------------------------------------------------------------

// CATEGORY FUNCTIONS BEGIN

/**
 * get all categories
 * @returns array
 * @return array of categories, or false on failure
 */
function reco_userapi_getallcats($args)
{
    extract($args);

    // Optional arguments.
    if (!isset($startnum)) {
        $startnum = 1;
    }
    if (!isset($numitems)) {
        $numitems = -1;
    }

    if ((!isset($startnum)) ||
        (!isset($numitems))) {
        pnSessionSetVar('errormsg', _MODARGSERROR);
        return false;
    }

    $items = array();

    // Security check - important to do this as early on as possible to 
    // avoid potential security holes or just too much wasted processing
    if (!pnSecAuthAction(0, 'Reco::', '::', ACCESS_READ)) {
        return $items;
    }

    // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
    // return arrays but we handle them differently.  For pnDBGetConn() we
    // currently just want the first item, which is the official database
    // handle.  For pnDBGetTables() we want to keep the entire tables array
    // together for easy reference later on
    list($dbconn) = pnDBGetConn();
    $pntable = pnDBGetTables();

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $recocattable = $pntable['recocategory'];
    $recocatcolumn = &$pntable['recocategory_column'];

    // Get items - the formatting here is not mandatory, but it does make the
    // SQL statement relatively easy to read.  Also, separating out the sql
    // statement from the SelectLimit() command allows for simpler debug
    // operation if it is ever needed
    $sql = "SELECT $recocatcolumn[id],
                   $recocatcolumn[category]
            FROM $recocattable
            ORDER BY $recocatcolumn[category]";
    $result = $dbconn->SelectLimit($sql, $numitems, $startnum-1);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        pnSessionSetVar('errormsg', _GETFAILED);
        return false;
    }

    // Put items into result array.  Note that each item is checked
    // individually to ensure that the user is allowed access to it before it
    // is added to the results array
    for (; !$result->EOF; $result->MoveNext()) {
        list($id, $name, $number) = $result->fields;
        if (pnSecAuthAction(0, 'Reco::', "$name::$id", ACCESS_READ)) {
            $items[] = array('id' => $id,
                             'name' => $name);
                          
        }
    }

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the items
    return $items;
}

/**
 * get a specific category
 * @param $args['id'] id of example item to get
 * @returns array
 * @return item array, or false on failure
 */
function reco_userapi_getcat($args)
{
    // Get arguments from argument array - all arguments to this function
    // should be obtained from the $args array, getting them from other places
    // such as the environment is not allowed, as that makes assumptions that
    // will not hold in future versions of PostNuke
    extract($args);

    // Argument check - make sure that all required arguments are present, if
    // not then set an appropriate error message and return
    if (!isset($id)) {
        pnSessionSetVar('errormsg', _MODARGSERROR);
        return false;
    }

    // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
    // return arrays but we handle them differently.  For pnDBGetConn() we
    // currently just want the first item, which is the official database
    // handle.  For pnDBGetTables() we want to keep the entire tables array
    // together for easy reference later on
    list($dbconn) = pnDBGetConn();
    $pntable = pnDBGetTables();

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules

    $recocattable = $pntable['recocategory'];
    $recocatcolumn = &$pntable['recocategory_column'];

    // Get item - the formatting here is not mandatory, but it does make the
    // SQL statement relatively easy to read.  Also, separating out the sql
    // statement from the Execute() command allows for simpler debug operation
    // if it is ever needed
    $sql = "SELECT $recocatcolumn[category]                   
            FROM $recocattable
            WHERE $recocatcolumn[id] = " . pnVarPrepForStore($id);
    $result = $dbconn->Execute($sql);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return false;
    }

    // Check for no rows found, and if so return
    if ($result->EOF) {
        return false;
    }

    // Obtain the item information from the result set
    list($name) = $result->fields;

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Security check - important to do this as early on as possible to avoid
    // potential security holes or just too much wasted processing.  Although
    // this one is a bit late in the function it is as early as we can do it as
    // this is the first time we have the relevant information
    if (!pnSecAuthAction(0, 'Reco::', "$name::$id", ACCESS_READ)) {
        return false;
    }

    // Create the item array
    $item = array('id' => $id,
                  'category' => $name);
              

    // Return the item array
    return $item;
}





// ITEM FUNCTIONS BEGIN

/**
 * get all items
 * @param getcat=category of items to get (optional)
 * @returns array
 * @return array of items, or false on failure
 */
function reco_userapi_getallitems($args)
{
    extract($args);

    // Optional arguments.
    if (!isset($startnum)) {
        $startnum = 1;
    }
    if (!isset($numitems)) {
        $numitems = -1;
    }
    if (!isset($getcat)) {
        $getcat=-1;
    }

    if ((!isset($startnum)) ||
	(!isset($getcat)) ||
        (!isset($numitems))) {
        pnSessionSetVar('errormsg', _MODARGSERROR);
        return false;
    }

    $items = array();

    // Security check - important to do this as early on as possible to 
    // avoid potential security holes or just too much wasted processing
    if (!pnSecAuthAction(0, 'Reco::', '::', ACCESS_READ)) {
        return $items;
    }

    // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
    // return arrays but we handle them differently.  For pnDBGetConn() we
    // currently just want the first item, which is the official database
    // handle.  For pnDBGetTables() we want to keep the entire tables array
    // together for easy reference later on
    list($dbconn) = pnDBGetConn();
    $pntable = pnDBGetTables();

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $recoitemtable = $pntable['recoitem'];
    $recoitemcolumn = &$pntable['recoitem_column'];
    $recocattable = $pntable['recocategory'];
    $recocatcolumn = &$pntable['recocategory_column'];

    // Get items - the formatting here is not mandatory, but it does make the
    // SQL statement relatively easy to read.  Also, separating out the sql
    // statement from the SelectLimit() command allows for simpler debug
    // operation if it is ever needed

    $sql = "SELECT $recoitemtable.$recoitemcolumn[id],
                   $recoitemtable.$recoitemcolumn[category],
                   $recoitemcolumn[desc1],
                   $recoitemcolumn[desc2],
                   $recoitemcolumn[imgsrc],
                   $recoitemcolumn[buyurl],
                   $recocattable.$recocatcolumn[category]
            FROM $recoitemtable, $recocattable
            WHERE $recocattable.$recocatcolumn[id]=$recoitemtable.$recoitemcolumn[category]";
    if ($getcat!=-1) {
            $sql=$sql . " AND $recoitemtable.$recoitemcolumn[category]=$getcat ";
    }
    $sql=$sql . " ORDER BY $recoitemcolumn[desc1]";
            
           
    $result = $dbconn->SelectLimit($sql, $numitems, $startnum-1);
    
    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        pnSessionSetVar('errormsg', _GETFAILED);
        return false;
    }
    
    // Put items into result array.  Note that each item is checked
    // individually to ensure that the user is allowed access to it before it
    // is added to the results array
    for (; !$result->EOF; $result->MoveNext()) {
        list($id, $category, $desc1, $desc2, $imgsrc, $buyurl, $catname) = $result->fields;
        if (pnSecAuthAction(0, 'Reco::', "$title::$id", ACCESS_READ)) {
            $items[] = array('id' => $id,
                             'category' => $category,
                             'desc1' => $desc1,
                             'desc2' => $desc2,
                             'imgsrc' => $imgsrc,
                             'buyurl' => $buyurl,
                             'catname' => $catname);
                          
        }
    }

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the items
    return $items;
}



/**
 * get a specific item
 * @param $args['id'] id of example item to get
 * @returns array
 * @return item array, or false on failure
 */
function reco_userapi_getitem($args)
{
    // Get arguments from argument array - all arguments to this function
    // should be obtained from the $args array, getting them from other places
    // such as the environment is not allowed, as that makes assumptions that
    // will not hold in future versions of PostNuke
    extract($args);

    // Argument check - make sure that all required arguments are present, if
    // not then set an appropriate error message and return
    if (!isset($id)) {
        pnSessionSetVar('errormsg', _MODARGSERROR);
        return false;
    }

    // Get datbase setup - note that both pnDBGetConn() and pnDBGetTables()
    // return arrays but we handle them differently.  For pnDBGetConn() we
    // currently just want the first item, which is the official database
    // handle.  For pnDBGetTables() we want to keep the entire tables array
    // together for easy reference later on
    list($dbconn) = pnDBGetConn();
    $pntable = pnDBGetTables();

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules

    $recoitemtable = $pntable['recoitem'];
    $recoitemcolumn = &$pntable['recoitem_column'];

    // Get item - the formatting here is not mandatory, but it does make the
    // SQL statement relatively easy to read.  Also, separating out the sql
    // statement from the Execute() command allows for simpler debug operation
    // if it is ever needed
    $sql = "SELECT $recoitemcolumn[desc1],
    			$recoitemcolumn[desc2],
    			$recoitemcolumn[category],
    			$recoitemcolumn[imgsrc],
    			$recoitemcolumn[buyurl]     
            FROM $recoitemtable
            WHERE $recoitemcolumn[id] = " . pnVarPrepForStore($id);
    $result = $dbconn->Execute($sql);


    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return false;
    }

    // Check for no rows found, and if so return
    if ($result->EOF) {
        return false;
    }

    // Obtain the item information from the result set
    list($desc1, $desc2, $category, $imgsrc, $buyurl) = $result->fields;

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Security check - important to do this as early on as possible to avoid
    // potential security holes or just too much wasted processing.  Although
    // this one is a bit late in the function it is as early as we can do it as
    // this is the first time we have the relevant information
    if (!pnSecAuthAction(0, 'Reco::', "$desc1::$id", ACCESS_READ)) {
        return false;
    }

    // Create the item array
    $item = array('id' => $id,
                  'desc1' => $desc1,
                  'desc2' => $desc2,
                  'category' => $category,
                  'imgsrc' => $imgsrc,
                  'buyurl' => $buyurl);
              

    // Return the item array
    return $item;
}

?>